/* Libtlen2 - multiplatform Tlen.pl library
 * Copyright (C) 2003  Libtlen Team
 *
 * $Id: libtlen2.h,v 1.58 2004/05/29 17:25:36 mati Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */


#ifndef _LIBTLEN2_H_
#define _LIBTLEN2_H_

#include <glib.h>
#include <gnet.h>
#include <libxml/parser.h>
#include <libxml/tree.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */



extern const guint tlen_major_version;
extern const guint tlen_minor_version;
extern const guint tlen_micro_version;
extern const guint tlen_interface_age;
extern const guint tlen_binary_age;

/**
 *  TLEN_CHECK_VERSION
 *  @major: Major version number
 *  @minor: Minor version number
 *  @micro: Micro version number
 *
 *  Checks if the version given is compatible with this version of the
 *  library.  For example, TLEN_CHECK_VERSION(1.2.3) would return TRUE
 *  if the version is 1.2.5, and FALSE if 1.1.0.  This can be used in
 *  build tests.
 *
 **/
#define TLEN_CHECK_VERSION(major,minor,micro)    \
    (TLEN_MAJOR_VERSION > (major) || \
     (TLEN_MAJOR_VERSION == (major) && TLEN_MINOR_VERSION > (minor)) || \
     (TLEN_MAJOR_VERSION == (major) && TLEN_MINOR_VERSION == (minor) && \
      TLEN_MICRO_VERSION >= (micro)))


#define TLEN_HUB "idi.tlen.pl"
#define TLEN_FALLBACK_SERVER "s1.tlen.pl"
#define TLEN_FALLBACK_PORT 443

#define TLEN_IO_READ_BUFFER_SIZE (1024)

/* debug levels */

/**
  * TLEN_DEBUG_NONE
  *
  * No information will be printed.
  **/
#define TLEN_DEBUG_NONE 0

/**
  * TLEN_DEBUG_NORMAL
  *
  * Only errors from library functions will be reported.
  **/
#define TLEN_DEBUG_NORMAL 1

/**
  * TLEN_DEBUG_HIGH
  *
  * Maximum debug level.
  **/
#define TLEN_DEBUG_HIGH 2

/* state of TlenConnection */

/**
  * TLEN_STATE_NOT_CONNECTED
  *
  * Not connected.
  **/
#define TLEN_STATE_NOT_CONNECTED 0
/**
  * TLEN_STATE_CONNECTING_HUB
  *
  * Connecting to hub.
  **/
#define TLEN_STATE_CONNECTING_HUB 1
/**
  * TLEN_STATE_CONNECTING
  *
  * Connecting to server.
  **/
#define TLEN_STATE_CONNECTING 2
/**
  * TLEN_STATE_CONNECTED_TO_HUB
  *
  * Connected to hub.
  **/
#define TLEN_STATE_CONNECTED_TO_HUB 3
/**
  * TLEN_STATE_CONNECTED
  *
  * Connected to server.
  **/
#define TLEN_STATE_CONNECTED 4
/**
  * TLEN_STATE_LOGGING
  *
  * Logging into server.
  **/
#define TLEN_STATE_LOGGING 5
/**
  * TLEN_STATE_LOGGED
  *
  * Logged into server.
  **/
#define TLEN_STATE_LOGGED 6
/**
  * TLEN_STATE_UNAVAILABLE
  *
  * Connection socket exists, but stream disconnected
  **/
#define TLEN_STATE_UNAVAILABLE 7
/**
  * TLEN_STATE_CRITICAL_FAILURE
  *
  * Protocol error or memory exhaustion.
  * This connection is no longer usable
  * and should be freed.
  **/


/* pubdir */

#define TLEN_PUBDIR_JOB_NONE 0
#define TLEN_PUBDIR_JOB_UCZEN 1
#define TLEN_PUBDIR_JOB_STUDENT 2
#define TLEN_PUBDIR_JOB_ROLNIK 3
#define TLEN_PUBDIR_JOB_MENEDZER 4
    // specjalista, wolny zawod
#define TLEN_PUBDIR_JOB_SPEC_WOL 5
    // urzednik, uslugi, administracja
#define TLEN_PUBDIR_JOB_UUA 6
#define TLEN_PUBDIR_JOB_BEZROBOTNY 7
    // emeryt, rencista
#define TLEN_PUBDIR_JOB_EMERYT 8
#define TLEN_PUBDIR_JOB_GOSPODYNI 9
#define TLEN_PUBDIR_JOB_NAUCZYCIEL 10
#define TLEN_PUBDIR_JOB_LEKARZ 11
#define TLEN_PUBDIR_JOB_INNY 12

// pcie

#define TLEN_PUBDIR_GENDER_NONE 0
#define TLEN_PUBDIR_GENDER_MALE 1
#define TLEN_PUBDIR_GENDER_FEMALE 2

// plany

#define TLEN_PUBDIR_WANT_NONE 0
#define TLEN_PUBDIR_WANT_CINEMA 1
//...trzeba dopisac

// szukam...

#define TLEN_PUBDIR_LOOK_NONE 0
#define TLEN_PUBDIR_LOOK_TALK 1
#define TLEN_PUBDIR_LOOK_FRIEND 2
#define TLEN_PUBDIR_LOOK_FLIRT 3
#define TLEN_PUBDIR_LOOK_LOVE 4

// Glos

#define TLEN_PUBDIR_VOICE_DISABLED 0
#define TLEN_PUBDIR_VOICE_ENABLED 1

// Widocznosc

#define TLEN_PUBDIR_VISIBLE_NO 0
#define TLEN_PUBDIR_VISIBLE_YES 1

// Status

#define TLEN_PUBDIR_STATUS_AVAILABLE 2
#define TLEN_PUBDIR_STATUS_CHATTY 3



typedef enum{
  TLEN_MESSAGE_TYPE_CHAT = 0,
  TLEN_MESSAGE_TYPE_NORMAL
} TlenMessageType;


/* events */

typedef enum{
  TLEN_EVENT_UNKNOWN          = 0,
  TLEN_EVENT_CONNECTED,
  TLEN_EVENT_CONNECTION_FAILED,
  TLEN_EVENT_DISCONNECTED,
  TLEN_EVENT_UNAUTHORIZED,
  TLEN_EVENT_LOGGED,
  TLEN_EVENT_PRESENCE,            /* TlenEventPresence  */
  TLEN_EVENT_ROSTER,              /* TlenEventRoster    */
  TLEN_EVENT_MESSAGE,             /* TlenEventMessage   */
  TLEN_EVENT_CRITICAL_FAILURE,
  TLEN_EVENT_PUBDIR,
  TLEN_EVENT_PUBDIR_CHANGED,
  TLEN_EVENT_SEARCH,
  TLEN_EVENT_NEWMAIL,
  TLEN_EVENT_WEBMSG,
  TLEN_EVENT_FILE,
  TLEN_EVENT_P2P,
  TLEN_EVENT_ADVERT,
  TLEN_EVENT_NOTIFY,
} TlenEventType;

#define TLEN_EVENT_PRESENCE(x) ((TlenEventPresence*)(x->event))
#define TLEN_EVENT_ROSTER(x) ((TlenEventRoster*)(x->event))
#define TLEN_EVENT_MESSAGE(x) ((TlenEventMessage*)(x->event))
#define TLEN_EVENT_PUBDIR(x) ((TlenEventPubdir*)(x->event))
#define TLEN_EVENT_SEARCH(x) ((TlenEventSearch*)(x->event))
#define TLEN_EVENT_NEWMAIL(x) ((TlenEventMessage*)(x->event))
#define TLEN_EVENT_WEBMSG(x) ((TlenEventMessage*)(x->event))
#define TLEN_EVENT_FILE(x) ((TlenEventFile*)(x->event))
#define TLEN_EVENT_P2P(x) ((TlenEventP2p*)(x->event))
#define TLEN_EVENT_ADVERT(x) ((TlenEventAdvert*)(x->event))
#define TLEN_EVENT_NOTIFY(x) ((TlenEventNotify*)(x->event))

/**
  * TlenPresenceStatus:
  *
  * Specifies status of presence.
  **/
typedef enum {
  TLEN_PRESENCE_AVAILABLE = 2,
  TLEN_PRESENCE_EXT_AWAY,
  TLEN_PRESENCE_AWAY,
  TLEN_PRESENCE_DND,
  TLEN_PRESENCE_CHATTY,
  TLEN_PRESENCE_INVISIBLE,
  TLEN_PRESENCE_UNAVAILABLE,
} TlenPresenceStatus;

/**
  * TlenPresenceType:
  *
  * Specifies type of presence.
  **/
typedef enum {
  TLEN_PRESENCE_STATUS = 0,
  TLEN_PRESENCE_SUBSCRIBE,
  TLEN_PRESENCE_UNSUBSCRIBE,
  TLEN_PRESENCE_UNSUBSCRIBED,
  TLEN_PRESENCE_SUBSCRIBED,
} TlenPresenceType;

/**
  * TlenErrorType:
  *
  * Specifies type of error if occurred.
  **/
/*typedef enum {
  TLEN_ERROR_TYPE_NONE = 0
} TlenErrorType;  - will we need it? */


/**
 *
 * Descriptions of events.
 *
 **/

typedef struct _TlenEvent TlenEvent;
struct _TlenEvent
{
  gpointer event;
  TlenEventType type;
};

typedef struct _TlenEventPresence TlenEventPresence;
struct _TlenEventPresence
{
  gchar *from;
  TlenPresenceStatus status;
  gchar *description;
  gchar *type;
};

typedef struct _TlenRosterItem TlenRosterItem;
struct _TlenRosterItem
{
  gchar *jid;
  gchar *subscription;
  gchar *ask;
  gchar *name;
  gchar *group;
};

typedef struct _TlenEventRoster TlenEventRoster;
struct _TlenEventRoster
{
  GSList *list;
};

typedef struct _TlenEventSearch TlenEventSearch;
struct _TlenEventSearch
{
  GSList *list;
};

typedef struct _TlenEventMessage TlenEventMessage;
struct _TlenEventMessage
{
  gchar *from;
  gchar *body;
  gchar *e; //subject in NEWMAIL event, email in WEBMSG event
  gchar *stamp;
  TlenMessageType type;
};

// Oznaczenia:
// *PS* - katalog i szukanie
// *S* - samo szukanie
// *SR* - szukanie i tylko wartosc zwracana
typedef struct
{
    gchar *firstname; // *PS* Imi
    gchar *lastname; // *PS* Nazwisko
    gchar *nick; // *PS* Nick
    gint gender; // *PS* Pe
    gchar *city; // *PS* Miasto
    gchar *email; // *PS* Email
    gint age; // *SR* ZWRACANY wiek
    gint age_min; // *S* min. wiek
    gint age_max; // *S* max. wiek
    gint look_for; // *PS* poszukuj...
    gchar *school; // *PS* szkoa
    gint job; // *PS* zawd
    gint status; // *S* stan
    gint voice; // *PS* z moliwoci rozmowy gosowej
    gchar *id; // *S* identyfikator
    gint plans; // *PS* plany
    gint birthyear; // *P* rok urodzenia
    gint visible; // *P* czy moj status ma byc widoczny? 
} TlenPubdirItem;

typedef struct
{
  TlenPubdirItem *item;
} TlenEventPubdir;

typedef struct
{
  gchar *id;
  gchar *filename;
  gchar *i;
  gsize length;
  gsize bytes; //bytes sent/read
  GTcpSocket *socket;
  gchar *buf; //buffer to read from/write to
  gchar *hostname;
  gint port;
} TlenP2p;

typedef enum{
  TLEN_P2P_UNKNOWN = 0,
  TLEN_P2P_ASK = 1, //ask for file upload
  TLEN_P2P_ERROR = 3, //some error
  TLEN_P2P_DENIED = 4, //user denied file
  TLEN_P2P_ACCEPTED = 5, //user accepted file
  TLEN_P2P_OPENED = 6, //p2p server opened by sender
  TLEN_P2P_CONNECT = 7, //p2p server opened by receiver
} TlenP2pErrorType;

typedef struct
{
  TlenP2pErrorType error;
  TlenP2p *p;
} TlenEventFile;

typedef struct
{
  GTcpSocket *socket;
  gchar *i;
} TlenEventP2p;

typedef struct
{
  gchar *name;
  gchar *url;
  gint width;
  gint height;
} TlenEventAdvert;

typedef enum
{
  TLEN_NOTIFY_ALERT = 0, 
  TLEN_NOTIFY_TYPING,
  TLEN_NOTIFY_STOPPED,
} TlenNotifyType;

typedef struct
{
  TlenNotifyType type;
  gchar *from;
} TlenEventNotify;

typedef struct _TlenConnection TlenConnection;

/**
  * TlenEventCallback:
  * @tc: connection.
  *
  * Callback executed when new event appeared.
  *
  **/
typedef void (*TlenEventCallback)(TlenConnection *tc);

struct _TlenConnection
{
  gchar *username;
  gchar *password;
  gchar *hostname;
  gchar *sid;
  guint port;
  GTcpSocket *socket;
  guchar state;
  TlenPresenceType status;
  gchar *description;
  GSList *events_list;
  TlenEventCallback event_cb;
  GString *buffer;
  /*TlenErrorType error;*/
  gboolean critical_failure;
  gboolean id_parsed;
};


/* initialization */
void tlen_init (void);

/* TlenConnection routines */
TlenConnection*
tlen_connection_new(const gchar *username,
                    const gchar *password);
gboolean
tlen_connection_open(TlenConnection *tc, TlenEventCallback event_cb);
gboolean
tlen_connection_close(TlenConnection *tc);
gboolean
tlen_connection_delete(TlenConnection *tc);
gboolean
tlen_connection_socket_writen (TlenConnection* tc, gpointer buffer,
                               gsize length, gsize* bytes_n);
gboolean
tlen_connection_socket_readline (TlenConnection* tc, gchar** buffer);
gboolean
tlen_connection_socket_readn (TlenConnection* tc, gpointer buffer,
                              gsize length, gsize* bytes_n);

/* some commands */
gboolean tlen_notify (TlenConnection *tc, const gchar *destination,
                      TlenNotifyType type);
gboolean tlen_presence (TlenConnection *tc, TlenPresenceType status,
                        const gchar *description);
gboolean tlen_roster_get (TlenConnection *tc);
gboolean tlen_message (TlenConnection *tc, const gchar *destination,
                       const gchar *message, TlenMessageType type);
TlenPresenceType tlen_presence_type (TlenEvent *ev);

/* roster management */
gboolean tlen_subscribe_request (TlenConnection *tc, const gchar *id);
gboolean tlen_subscribe_accept (TlenConnection *tc, const gchar *id);
gboolean tlen_unsubscribe_accept (TlenConnection *tc, const gchar *id);
gboolean tlen_unsubscribe_request (TlenConnection *tc, const gchar *id);
gboolean tlen_contact_remove (TlenConnection *tc, const gchar *id);
gboolean tlen_contact_add (TlenConnection *tc, const gchar *id,
                           const gchar *name, const gchar *group);


/* search and pubdir */
gboolean tlen_pubdir_get (TlenConnection *tc);
gboolean tlen_pubdir_set (TlenConnection *tc, TlenPubdirItem *item);
void tlen_pubdir_item_free (TlenPubdirItem *item);
TlenPubdirItem *tlen_pubdir_item_new (void);
gboolean tlen_pubdir_search (TlenConnection *tc, TlenPubdirItem *item);

/* p2p connections */
gboolean tlen_p2p_sendfile (TlenConnection *tc, const gchar *id,
                            const gchar *path, gchar **i);
gboolean tlen_p2p_denyfile (TlenConnection *tc, const gchar *id,
                            const gchar *i);
gboolean tlen_p2p_open (TlenConnection *tc, const gchar *id,
                        const gchar *i);
gboolean tlen_p2p_connect (TlenConnection *tc, const gchar *hostname,
                           gint port, const gchar *i);

/* TlenEvent routines */
void tlen_event_free (TlenEvent *ev);
TlenEvent *tlen_event_get (TlenConnection *tc);

/* some utilities */
guint tlen_strlen (const gchar *s);
gchar *tlen_encode (const gchar *what);
gchar *tlen_decode(const gchar *what);
gint tlen_strcmp (const gchar *s1, const gchar *s2);

/* debug system */
void tlen_debug_set(guint level);
guint tlen_debug_get(void);
void _tlen_debug(int debug_level, const gchar *name, gchar *format, ...);
/**
  * tlen_debug:
  * @...: available format is %s for string, %c for char and %d for int,
  *       \n is available.
  *
  * Prints debug info if debug level is greater then #TLEN_DEBUG_NONE
  **/
#define tlen_debug(...) if(tlen_debug_get () > TLEN_DEBUG_NONE) \
  _tlen_debug(TLEN_DEBUG_NORMAL, __func__, __VA_ARGS__)
/**
  * tlen_debug_high:
  * @...: same as for #tlen_debug.
  *
  * Prints debug info if debug level is greater then #TLEN_DEBUG_NORMAL
  **/
#define tlen_debug_high(...) if(tlen_debug_get () > TLEN_DEBUG_NORMAL) \
  _tlen_debug(TLEN_DEBUG_HIGH, __func__, __VA_ARGS__)

#ifdef __WIN32__
  #define xmlFree xmlFreeFunc
#endif


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* _LIBTLEN2_H */
